/*
 * Copyright 2016 Fairphone B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.fairphone.hiccup.app;

import android.content.Context;
import android.content.SharedPreferences;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.preference.PreferenceManager;
import android.support.annotation.NonNull;

import com.fairphone.hiccup.app.R;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

/**
 * This class communicates with the Hiccup Webservice.
 */
public class HiccupAPIClient
{
    String mToken;
    String mUuid;
    Context mContext;

    public class SendingOverMobileDataProhibitedException extends Exception {}
    public class HiccupServerCommunicationException extends Exception {}
    public class HiccupServerProtocolViolationException extends Exception {}

    HiccupAPIClient(Context ctx) throws SendingOverMobileDataProhibitedException,
            HiccupServerProtocolViolationException, HiccupServerCommunicationException {
        this.mContext = ctx;
        if (!isRegistered()) {
            registerDevice();
        } else {
            mToken = getAPITokenFromPreferences();
            mUuid = getUuidFromPreferences();
        }
    }

    /**
     * Will try to upload a report. Once the upload finished the report will be marked
     * as uploaded in the database.
     * @param id the id of the report to be uploaded
     */
    void uploadReport(long id)
            throws HiccupServerCommunicationException,
            SendingOverMobileDataProhibitedException,
            HiccupServerProtocolViolationException {
        HiccupReportDB hiccupReportDB = new HiccupReportDB(mContext);
        uploadReport(hiccupReportDB.getReport(id));
    }

    void uploadReport(HiccupReport report)
            throws HiccupServerCommunicationException,
            SendingOverMobileDataProhibitedException,
            HiccupServerProtocolViolationException {
        if (report == null) {
            return;
        }
        switch (report.getReportType()) {
            case HiccupReport.TYPE_CRASH_REPORT:
            case HiccupReport.TYPE_FAKE_REPORT:
                uploadCrashReport(report);
                break;
            case HiccupReport.TYPE_HEARTBEAT:
                uploadHeartbeatReport(report);
                break;
        }
    }

    @NonNull
    private JSONObject getCrashreportJsonObject(HiccupReport report) throws JSONException {
        JSONObject requestData = new JSONObject();
        requestData.put("uuid", mUuid);
        requestData.put("aux_data", report.getAuxData());
        requestData.put("uptime", report.getUptime());
        requestData.put("power_on_reason", report.getPowerOnReason());
        requestData.put("power_off_reason", report.getPowerOffReason());
        requestData.put("boot_reason", report.getBootReason());
        requestData.put("build_fingerprint", report.getBuildFingerPrint());
        requestData.put("date", new java.sql.Timestamp(report.getDate().getTime()).toString());
        requestData.put("app_version", String.valueOf(HiccupUtil.getVersionCode(mContext)));
        requestData.put("is_fake_report",
                HiccupReport.TYPE_FAKE_REPORT.equals(report.getReportType()));
        return requestData;
    }

    @NonNull
    private JSONObject getHeartbeatJsonObject(HiccupReport report) throws JSONException {
        JSONObject requestData = new JSONObject();
        requestData.put("uuid", mUuid);
        requestData.put("uptime", report.getUptime());
        requestData.put("build_fingerprint", report.getBuildFingerPrint());
        requestData.put("date", new java.sql.Timestamp(report.getDate().getTime()).toString());
        requestData.put("app_version", String.valueOf(HiccupUtil.getVersionCode(mContext)));
        return requestData;
    }

    /**
     * Creates the crashreport on the server and returns the device local ID of the
     * server side report.
     * @param report The crash report to be created on the server
     * @return the id on the server side
     * @throws HiccupServerCommunicationException Could not communicate with the server
     * @throws HiccupServerProtocolViolationException Server sent an unexpected Response
     */
    private int createCrashReport(HiccupReport report)
            throws SendingOverMobileDataProhibitedException,
            HiccupServerCommunicationException,
            HiccupServerProtocolViolationException {
        checkDataConnection();
        JSONObject requestData;
        try {
            requestData = getCrashreportJsonObject(report);
        } catch (JSONException e) {
            e.printStackTrace();
            /* TODO: Catch this exception properly There is something wrong with the report?
             * For now we throw an HiccupServerCommunicationException
             */
            throw new HiccupServerCommunicationException();
        }
        JSONObject responseJSON = performHiccupCall("api/v1/crashreports/", requestData);
        if (! responseJSON.has("device_local_id") )
            throw new HiccupServerProtocolViolationException();
        try {
            return Integer.valueOf(responseJSON.getString("device_local_id"));
        } catch (JSONException e) {
            HiccupUtil.LOG(e.getMessage());
            throw new HiccupServerProtocolViolationException();
        }
    }

    /**
     * Utility function to make a API call to the Hiccup Server.
     * @param hiccupAPIEndPoint The endpoint the call should go to.
     * @param requestData Contains the request parameters.
     * @return A JSONObject containing the response of the Hiccup server.
     * @throws HiccupServerCommunicationException
     */
    private JSONObject performHiccupCall(String hiccupAPIEndPoint, JSONObject requestData)
            throws HiccupServerCommunicationException, SendingOverMobileDataProhibitedException {
        checkDataConnection();
        try {
            URL url= new URL(getHiccupURL(),hiccupAPIEndPoint);
            Request.Builder requestBuilder = new Request.Builder();
            OkHttpClient client = new OkHttpClient();
            requestBuilder.url(url);
            /* if we are registered use our token for authentication */
            if (isRegistered()) {
                requestBuilder.addHeader("Authorization", "Token " + mToken);
            }
            requestBuilder.post(RequestBody.create(MediaType.parse("application/json"),
                    requestData.toString()));
            Response response = client.newCall(requestBuilder.build()).execute();
            if (!response.isSuccessful()) {
                throw new HiccupServerCommunicationException();
            }
            return new JSONObject(response.body().string());
        } catch(IOException | JSONException e) {
            throw new HiccupServerCommunicationException();
        }
    }


    /**
     * Uploads the logfiles associated with a report. The logfiles are uploaded as zip archive.
     * If the method did not throw an exception the upload was successful. The upload happens
     * synchronously. Do never call from an UI thread!
     *
     * @param report The report of which the logfiles shall be uploaded
     * @throws HiccupServerCommunicationException Thrown if something goes wrong along the way.
     *         If thrown the logfile was not uploaded and Hiccup should retry.
     */
    private void uploadLogFiles(HiccupReport report) throws HiccupServerCommunicationException {
        OkHttpClient client = new OkHttpClient();
        Request.Builder requestBuilder = new Request.Builder();
        MultipartBody.Builder multipartBodyBuilder = new MultipartBody.Builder();
        File file = new File(report.getLogFilesPath());
        multipartBodyBuilder.addFormDataPart("file", file.getName(),
                RequestBody.create(MediaType.parse("application/zip"), file));
        try {
            requestBuilder.url(new URL(
                    getHiccupURL(),
                    "api/v1/devices/"
                            + mUuid
                            + "/crashreports/"
                            + report.getServerSideLocalId()
                            + "/logfile_put/logfile_archive.zip/"));
            requestBuilder.addHeader("Authorization", "Token " + mToken);
            requestBuilder.post(multipartBodyBuilder.build());
            Response response = client.newCall(requestBuilder.build()).execute();
            if (response.isSuccessful()) {
                return;
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        throw new HiccupServerCommunicationException();
    }

    private void uploadCrashReport(HiccupReport report)
            throws HiccupServerCommunicationException,
            SendingOverMobileDataProhibitedException,
            HiccupServerProtocolViolationException {
        /* Do we have to create the report on server side? */
        if (!report.existsOnServerSide()) {
            int deviceLocalID = createCrashReport(report);
            report.setServerSideLocalId(deviceLocalID);
            report.save(mContext);
        }
        /* Do we have to upload log files? */
        if (report.hasLogFiles()) {
            uploadLogFiles(report);
        }
        report.setUploadCompleted(true);
        report.save(mContext);

    }

    private void uploadHeartbeatReport(HiccupReport report)
            throws HiccupServerProtocolViolationException,
            HiccupServerCommunicationException,
            SendingOverMobileDataProhibitedException {
        JSONObject requestData;
        try {
            requestData = getHeartbeatJsonObject(report);
        } catch (JSONException e) {
            e.printStackTrace();
            /* TODO: Catch this exception properly There is something wrong with the report?
             * For now we throw an HiccupServerCommunicationException
             */
            throw new HiccupServerCommunicationException();
        }
        JSONObject responseJSON = performHiccupCall("api/v1/heartbeats/", requestData);
        if (!responseJSON.has("device_local_id") )
            throw new HiccupServerProtocolViolationException();
        report.setUploadCompleted(true);
        report.save(mContext);
    }

    private boolean isRegistered() {
        /* if no UUID is stored in the preferences we assume Hiccup is uninitialized */
        return (!"".equals(getUuidFromPreferences()));
    }

    private void checkDataConnection() throws SendingOverMobileDataProhibitedException {
        if (HiccupSettings.isMobileDataAllowed(mContext))
            return;
        ConnectivityManager connMgr = (ConnectivityManager)
                mContext.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connMgr.getNetworkInfo(ConnectivityManager.TYPE_WIFI);
        if (!networkInfo.isConnected()) {
            throw new SendingOverMobileDataProhibitedException();
        }
    }

    private void registerDevice()
            throws SendingOverMobileDataProhibitedException,
            HiccupServerProtocolViolationException,
            HiccupServerCommunicationException {
        JSONObject requestData;
        try {
            requestData = new JSONObject();
            requestData.put("board_date",
                    new java.sql.Timestamp(HiccupUtil.getBoardDate().getTime()).toString());
            requestData.put("chipset", HiccupUtil.getDeviceProcessorInfo());

        } catch (JSONException e) {
            e.printStackTrace();
            /* TODO: Catch this exception properly There is something wrong with the report?
             * For now we throw an HiccupServerCommunicationException
             */
            throw new HiccupServerCommunicationException();
        }
        JSONObject responseJSON = performHiccupCall("api/v1/devices/register/", requestData);
        try {
            mUuid  = responseJSON.getString("uuid");
            mToken = responseJSON.getString("token");

        } catch (JSONException e) {
            e.printStackTrace();
            throw new HiccupServerProtocolViolationException();
        }
        setUUIDToPreference(mUuid);
        setAPITokenToPreference(mToken);

    }

    public String getUUID() {return mUuid;}

    private URL getHiccupURL() {
        String ret = "https://api.fairphone.software/hiccup/";
        try {
            Process proc = Runtime.getRuntime().exec(new String[]{"/system/bin/getprop", "persist.fp.hiccup.url"});
            BufferedReader reader = new BufferedReader(new InputStreamReader(proc.getInputStream()));
            String output = reader.readLine();
            if (!output.isEmpty()) {
                ret = output;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            return new URL(ret);
        } catch (MalformedURLException e) {
            e.printStackTrace();
            return null;
        }
    }

    private String getAPITokenFromPreferences() {
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(mContext);
        return preferences.getString(mContext.getString(R.string.PREF_KEY_API_TOKEN), "");
    }

    private void setAPITokenToPreference(String newVal) {
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(mContext);
        SharedPreferences.Editor editor = preferences.edit();
        editor.putString(mContext.getString(R.string.PREF_KEY_API_TOKEN), newVal);
        editor.apply();
    }

    private String getUuidFromPreferences() {
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(mContext);
        return preferences.getString(mContext.getString(R.string.PREF_KEY_UUID), "");
    }

    private void setUUIDToPreference(String newVal) {
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(mContext);
        SharedPreferences.Editor editor = preferences.edit();
        editor.putString(mContext.getString(R.string.PREF_KEY_UUID), newVal);
        editor.apply();
    }
}
